#include <pic.h>
#include "LedDriver.h"

// Device configuration
__CONFIG(UNPROTECT& WDTDIS & MCLREN & 
DEBUGDIS & UNPROTECT & UNPROTECT & BORDIS & PWRTEN & INTIO);

void InitDevice();

#define     MODE_TIME       0x01
#define     MODE_SET        0x02
#define     MODE_DBG        0x04
unsigned char displayMode;

unsigned char second;
unsigned char minute;
unsigned char hour;
unsigned char hideSecond;
unsigned char keyCntTmp;
unsigned char keyCnt;

extern const unsigned char dirTable[16];
unsigned char encValue;
// ת
#define     POS     1
// ת
#define     NEG     2
// Ч
#define     NAC     0

/// жϴ
void interrupt ISR()
{
    if(T0IF){
        // Timer0 ж Լ8msһΣˢʾⰴ
        T0IF=0;
        TMR0 = 0xC0;
        if(displayMode & MODE_TIME){
            // ˢʾ
            static unsigned char scanIndex = 0;
            TRISC = (unsigned char)dispBuffer[scanIndex].dirData;
            TRISA = (unsigned char)(dispBuffer[scanIndex].dirData>>8);
            PORTC = (unsigned char)dispBuffer[scanIndex].ioData;
            PORTA = (unsigned char)(dispBuffer[scanIndex].ioData>>8);
            scanIndex++;
            if(scanIndex >=3)scanIndex = hideSecond;
        }
        if(RB1){
            // ⰴ
            if(keyCntTmp){
                keyCnt = keyCntTmp;
            }
            keyCntTmp = 0;
        }else{
            if(keyCntTmp & 0x80){
            }else{
                keyCntTmp++;
            }
        }
    }
    if(TMR1IF){
        // Timer1 ж 1sһΣˢʾⰴ
        TMR1IF = 0;
        TMR1H|=0x80;
        if(displayMode & MODE_SET){
            //  ˸
            hideSecond = (hideSecond+1) & 1;
            second = 0;
            FormatSecond(second);
        }else{
            //  ʱ
            second++;
            if(second >= 60){
                second = 0;
                minute++;
                if(minute >= 60){
                    minute = 0;
                    hour++;
                    if(hour >= 12){
                        hour = 0;
                    }
                    FormatHour(hour);
                }
                FormatMinute(minute);
            }
            FormatSecond(second);
        }
    }
    if(RBIF){
        // RB6RB7仯жϣ
        unsigned char encDir;
        encValue |= (PORTB & 0b11000000);
        RBIF = 0;
        if(displayMode & MODE_SET){
            // λ
            encDir = dirTable[encValue>>4];
            encValue>>=2;
            if(encDir & POS){
                //  ˳ʱת
                minute++;
                if(minute >= 60){
                    minute = 0;
                    hour++;
                    if(hour >= 12){
                        hour = 0;
                    }
                    FormatHour(hour);
                }
                FormatMinute(minute);
            }else if(encDir & NEG){
                //  ʱת
                /// Negative rotate
                if(minute == 0){
                    minute = 60;
                    if(hour == 0){
                        hour = 12;
                    }
                    hour--;
                    FormatHour(hour);
                }
                minute--;
                FormatMinute(minute);
            }
        }
    }
}

void main()
{
    // ʼ
    displayMode = MODE_TIME;
    second = 0;
    minute = 0;
    hour = 0;
    hideSecond = 0;
    FormatHour(hour);
    FormatMinute(minute);
    FormatSecond(second);
    keyCntTmp = 0;
    keyCnt = 0;
    // ʼ
    InitDevice();
    while(1){
        if(keyCnt>10){
            // а£лģʽ
            keyCnt = 0;
            if(displayMode & MODE_SET){
                displayMode &= (~MODE_SET);
            }else{
                displayMode |= MODE_SET;
            }    
        }
    }
}

// Initial device
void InitDevice()
{
    // Internal 8MHz clock
    OSCCON = 0b01110001;
    
    //1:256 Timer0
    OPTION= 0B00000111; // enable RB wake pull up
    // 5ms = 5000us 256*20 = 5120  256-20=236;
    TMR0=236;
    
    // Config PORTC, all output
    TRISC = 0b0000000;
    PORTC = 0x00;
    LCDCON = 0;
    
    // Config PORTB, <7:0> input, wake pull up enable
    PORTB = 0;
    TRISB = 0xFF;
    WPUB = 0xFF;
    IOCB = 0b11000000; // <7:6> interrupt on change enable
    RBIE = 1; // Enable interrupt on change of PORTB
    
    // Config PORTA
    CMCON0 = 0x07;
    ANSEL = 0x03;  // <1:0> set as analog input
    PORTA = 0;
    TRISA = 0xFF;
    
    //Enable GI, Enable TMR0 Int, PEIE enable
    INTCON=0B11100000;
    //Enable recive Int
    //Set TMR1
    T1CON=0B00001111;
    //Enable TMR1 Int
    TMR1H|=0x80;
    TMR1IE=1;
}

// Direction table for the encoder
/**
  Encoder output format:
   Clockwise:           00 01 11 10 00 01 11 10 ...
   counterClockwise:    00 10 11 01 00 10 11 01 ...
 */
const unsigned char dirTable[16] = 
{
        NAC,    // 0000
        NEG,    // 0001
        POS,    // 0010
        NAC,    // 0011
        POS,    // 0100
        NAC,    // 0101
        NAC,    // 0110
        NEG,    // 0111
        NEG,    // 1000
        NAC,    // 1001
        NAC,    // 1010
        POS,    // 1011
        NAC,    // 1100
        POS,    // 1101
        NEG,    // 1110
        NAC,    // 1111
};

